/*!
 * @file    uwb_SmartField_IO.c
 * @author  Kryštof Vaněk <krystof.vanek@gmail.com>
 * @version 1.0
 * @date    2021-02-15
 * @brief   SmartField IO driver prototype
 *
 * @attention <h2><center>&copy; Copyright (c) 2021 University of West Bohemia in Pilsen All rights reserved.</center></h2>
 * @attention <h2><center>Use of this source code without author permission and license agreement is prohibited. For license agreement contact the author.</center></h2>
 * @attention <h2><center>Tento zdrojový kód nelze využívat bez souhlasu autora a dohodě o licenčních podmínkách. Pro případné licencování kontaktujte autora.</center></h2>
 *
 * Developed by:<br>
 * University of West Bohemia<br>
 * Department of Technologies and Measurement<br>
 * www.zcu.cz
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
 * OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
 */

#include "uwb_SmartField.h"
#include "string.h"

SmartField_IO_TerminalColors_t terminalColors=
	{
		.reset		="\033[0m",
		.red		="\033[0;31m",
		.green		="\033[0;32m",
		.blue		="\033[0;34m",
		.cyan		="\033[0;36m",
		.magenta	="\033[0;35m",
		.yellow		="\033[0;33m",
		.black		="\033[0;30m",
		.white		="\033[0;37m",
	};

#ifndef __weak
  #define __weak  __attribute__((weak))
#endif /* __weak */

__weak SmartField_IO_Time_t SmartField_IO_GetTime(){
	SmartField_IO_Time_t sSFIO_Time={0};
	return sSFIO_Time;
}

__weak void SmartField_IO_TerminalTimeStamp(){
	SmartField_IO_Time_t sSFIO_Time = SmartField_IO_GetTime();
	printf("%02d.%02d.%4d %02d:%02d:%02d ",
			sSFIO_Time.bDay, sSFIO_Time.bMonth,
			sSFIO_Time.bYear + 2000, sSFIO_Time.bHours,
			sSFIO_Time.bMinutes,	sSFIO_Time.bSeconds);
}

/**
 * @brief Turn on Status LED
 */
__weak SmartField_IO_Status_t SmartField_IO_LED_STS_ON(){
	return SFIO_NOT_IMPLEMENTED;
}
/**
 * @brief Turn off Status LED
 */
__weak SmartField_IO_Status_t SmartField_IO_LED_STS_OFF(){
	return SFIO_NOT_IMPLEMENTED;
}
/**
 * @brief Turn on Probe LED
 */
__weak SmartField_IO_Status_t SmartField_IO_LED_PRB_ON(){
	return SFIO_NOT_IMPLEMENTED;
}
/**
 * @brief Turn off Probe LED
 */
__weak SmartField_IO_Status_t SmartField_IO_LED_PRB_OFF(){
	return SFIO_NOT_IMPLEMENTED;
}
/**
 * @brief Turn on Error LED
 */
__weak SmartField_IO_Status_t SmartField_IO_LED_ERR_ON(){
	return SFIO_NOT_IMPLEMENTED;
}
/**
 * @brief Turn off Error LED
 */
__weak SmartField_IO_Status_t SmartField_IO_LED_ERR_OFF(){
	return SFIO_NOT_IMPLEMENTED;
}

/**
 * @brief Turn on Ident bus VDD
 *
 * Powers up identification EEPROMs.
 * Enables module identification memory readout
 *
 */
__weak SmartField_IO_Status_t SmartField_IO_VDD_Ident_ON(){
	return SFIO_NOT_IMPLEMENTED;
}
/**
 * @brief Turn off Ident bus VDD
 *
 * Shuts down identification EEPROMs.
 *
 */
__weak SmartField_IO_Status_t SmartField_IO_VDD_Ident_OFF(){
	return SFIO_NOT_IMPLEMENTED;
}

/**
 * @brief Turn on analog power suppy.
 */
__weak SmartField_IO_Status_t SmartField_IO_VCC_ON(){
	return SFIO_NOT_IMPLEMENTED;
}
/**
 * @brief Turn off analog power suppy.
 */
__weak SmartField_IO_Status_t SmartField_IO_VCC_OFF(){
	return SFIO_NOT_IMPLEMENTED;
}

/**
 * @brief Turn on power to waveform generator PLL
 *
 * @todo This is module specific function, shouldn't be in general module.
 */
__weak SmartField_IO_Status_t SmartField_IO_VDD_PLL_ON(){
	return SFIO_NOT_IMPLEMENTED;
	/* TODO This is module specific function, shouldn't be in general module */
}
/**
 * @brief Turn off power to waveform generator PLL
 *
 * @todo This is module specific function, shouldn't be in general module.
 */
__weak SmartField_IO_Status_t SmartField_IO_VDD_PLL_OFF(){
	return SFIO_NOT_IMPLEMENTED;
	/* TODO This is module specific function, shouldn't be in general module */
}

/**
 * @brief Enable waveform generator analog ouput
 */
__weak SmartField_IO_Status_t SmartField_IO_GEN_Enable(){
	return SFIO_NOT_IMPLEMENTED;
}
/**
 * @brief Disable waveform generator analog ouput
 */
__weak SmartField_IO_Status_t SmartField_IO_GEN_Disable(){
	return SFIO_NOT_IMPLEMENTED;
}

/**
 * @brief Activate impedance measurement module channel 1
 */
__weak SmartField_IO_Status_t SmartField_IO_CH1_Enable(){
	return SFIO_NOT_IMPLEMENTED;
}
/**
 * @brief Deactivate impedance measurement module channel 1
 */
__weak SmartField_IO_Status_t SmartField_IO_CH1_Disable(){
	return SFIO_NOT_IMPLEMENTED;
}

/**
 * @brief Activate impedance measurement module channel 2
 */
__weak SmartField_IO_Status_t SmartField_IO_CH2_Enable(){
	return SFIO_NOT_IMPLEMENTED;
}
/**
 * @brief Deactivate impedance measurement module channel 2
 */
__weak SmartField_IO_Status_t SmartField_IO_CH2_Disable(){
	return SFIO_NOT_IMPLEMENTED;
}

/**
 * @brief Enable internal I2C bus Pull-UPs
 *
 * Enable Pull-Up resistors (MCU external) on I2C bus dedicated to system internal communication.
 */
__weak SmartField_IO_Status_t SmartField_IO_I2C_Enable(){
	return SFIO_NOT_IMPLEMENTED;
}
/**
 * @brief Disable internal I2C bus Pull-UPs
 *
 * Disable Pull-Up resistors (MCU external) on I2C bus dedicated to system internal communication.
 */
__weak SmartField_IO_Status_t SmartField_IO_I2C_Disable(){
	return SFIO_NOT_IMPLEMENTED;
}

/**
 * @brief Read ADC data from parallel bus
 */
__weak SmartField_IO_Status_t SmartField_IO_SampleData(uint16_t* pBuff, uint32_t dataLength){
	return SFIO_NOT_IMPLEMENTED;
}

/**
 * @brief Write binary file header to SD card
 */
__weak SmartField_IO_Status_t SmartField_IO_SDcard_writeHeader(SmartField_Settings_t* settings){
	return SFIO_NOT_IMPLEMENTED;
}
/**
 * @brief Write data vectors to SD card
 */
__weak SmartField_IO_Status_t SmartField_IO_SDcard_writeData(uint8_t *pBuff, uint32_t dataLength){
	return SFIO_NOT_IMPLEMENTED;
}
/**
 * @brief Finalise SD card filesystem operations
 */
__weak SmartField_IO_Status_t SmartField_IO_SDcard_finalise(){
	return SFIO_NOT_IMPLEMENTED;
}
