#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "smartfield_fileloader.h"
#include "smartfield_simulator.h"

#include <QListWidget>
#include <QMainWindow>

QT_BEGIN_NAMESPACE
namespace Ui {
class MainWindow;
}
QT_END_NAMESPACE

class MainWindow : public QMainWindow {
    Q_OBJECT

public:
    MainWindow(QWidget* parent = nullptr);
    ~MainWindow();
    void closeEvent(QCloseEvent* event) override;

public slots:
    void openFiles();

private slots:
    void on_filesList_currentItemChanged(QListWidgetItem* current, QListWidgetItem* previous);
    void closeChartWindows();
    void rememberUIDCalib();
    void rawDataChart(int measurement);
    void filteredDataChart(int measurement);
    void treeWidgetItemExpanded(QTreeWidgetItem* item);

private:
    void recallUIDCalib();
    Ui::MainWindow* ui;
    void fillTableOfResults();
    void fillTableOfResults_DFT();
    SmartField_FileLoader& sfLoader = SmartField_FileLoader::getInstance();
    QList<QMainWindow*>* chartWindows = new QList<QMainWindow*>;
    QSignalMapper* signalMapperRawData = new QSignalMapper(this);
    QSignalMapper* signalMapperUTIT = new QSignalMapper(this);
    SmartField_Simulator& sfSIM = SmartField_Simulator::getInstance();
};

#endif // MAINWINDOW_H
