#include "smartfield_simulator.h"
#include "SmartField/uwb_SmartField.h"
#include "SmartField/uwb_SmartField_core.h"

SmartField_Handle_t hSmartField = {};

void SmartField_Simulator::setSettings(SmartField_Settings_t settings)
{
    hSmartField.Settings = settings;
}

uint32_t measurement = 0;
uint32_t sampleCnt = 0;

void SmartField_Simulator::evaluateAll()
{
    hSmartField.Settings = {};
    hSmartField.Settings.phiCalibMultiplier = 1;
    hSmartField.Settings.zCalibMultiplier = 1;

    if (sfLoader.getHeaderVersion() == 1) {
        hSmartField.Settings.phiCalibOffset = -9.012580734120409;
    }

    if (sfLoader.getUID() != "45-00-1d-00-03-50-50-53-42-38-34-20") {
        hSmartField.Settings.Zsns = 69.703537726290079;
    } else {
        hSmartField.Settings.Zsns = 49.991119722379;
    }

    hSmartField.Settings.normalisedFrequency = sfLoader.getNormalisedFreqeuncy();

    for (; measurement < (uint32_t)sfLoader.voltage.length(); measurement++) {
        SmartField_Core_SampleData();
        SmartField_Core_ProcessData(&hSmartField.Settings);
        SmartField_Core_DumpData(&getFilteredData);
        //SmartField_DumpRawData(&getRawData);
        results.append(*SmartField_Core_GetResult());
        SmartField_Core_ProcessData_DFT(&hSmartField.Settings);
        results_DFT.append(*SmartField_Core_GetResult());
    }
}

void SmartField_Simulator::reset()
{
    hSmartField.Settings = {};
    voltage.clear();
    current.clear();
    results.clear();
    results_DFT.clear();
    measurement = 0;
    sampleCnt = 0;
}

extern "C" void getFilteredData(float* fVoltage, float* fCurrent, uint32_t dataLength)
{
    QVector<float> tmpVoltage, tmpCurrent;
    for (unsigned i = 0; i < dataLength; i++) {
        tmpVoltage.append(fVoltage[i]);
        tmpCurrent.append(fCurrent[i]);
    }
    SmartField_Simulator* hSim = &SmartField_Simulator::getInstance();
    hSim->voltage.append(tmpVoltage);
    hSim->current.append(tmpCurrent);
}

extern "C" void getRawData(void* pData, uint32_t dataLength)
{
    QVector<float> tmpVoltage, tmpCurrent;
    SmartField_Sample_t* sample;
    sample = (SmartField_Sample_t*)pData;
    for (unsigned i = 0; i < dataLength; i++) {
        tmpVoltage.append(sample[i].voltage);
        tmpCurrent.append(sample[i].current);
    }
    SmartField_Simulator* hSim = &SmartField_Simulator::getInstance();
    hSim->voltage.append(tmpVoltage);
    hSim->current.append(tmpCurrent);
}

extern "C" uint16_t getLegacySample()
{
    static uint8_t lastMeasurement = 0;
    if (lastMeasurement != measurement) {
        sampleCnt = 0;
        lastMeasurement = measurement;
    }

    static SmartField_FileLoader* hSFLoader = &SmartField_FileLoader::getInstance();
    SmartField_Sample_t sample;
    sample.voltage = hSFLoader->voltage.at(measurement).at(sampleCnt);
    sample.current = hSFLoader->current.at(measurement).at(sampleCnt);
    sampleCnt++;
    return *((uint16_t*)&sample);
}

extern "C" SmartField_IO_Status_t SmartField_IO_SampleData(uint16_t* pBuff, uint32_t dataLength)
{
    for (uint32_t sample = 0; sample < dataLength; sample++) {
        pBuff[sample] = getLegacySample();
    }
    return SFIO_OK;
}
