#ifndef SMARTFIELD_SIMULATOR_H
#define SMARTFIELD_SIMULATOR_H

#include "SmartField/uwb_SmartField.h"
#include "smartfield_fileloader.h"

class SmartField_Simulator {
public:
    static SmartField_Simulator& getInstance()
    {
        static SmartField_Simulator instance; // Guaranteed to be destroyed, instantiated on first use.
        return instance;
    }
    /* Prevent copies of singleton */
    SmartField_Simulator(SmartField_Simulator const&) = delete;
    void operator=(SmartField_Simulator const&) = delete;

    void setInputFile(QFile inFile);
    void setSettings(SmartField_Settings_t settings);
    void evaluateAll();
    void reset();

    QVector<QVector<float>> voltage;
    QVector<QVector<float>> current;
    QVector<SmartField_Result_t> results;
    QVector<SmartField_Result_t> results_DFT;

private:
    SmartField_Simulator() {};
    SmartField_FileLoader& sfLoader = SmartField_FileLoader::getInstance();
};

extern "C" void getFilteredData(float* fVoltage, float* fCurrent, uint32_t dataLength);
extern "C" void getRawData(void* pData, uint32_t dataLength);
#endif // SMARTFIELD_SIMULATOR_H
